@@ -50,9 +50,9 @@ static void google_result_fetch(SearchResult *result);
 #endif
 
 /* callbacks */
-static void google_entry_activate_cb(GtkEntry *widget, Google *google);
-static void google_icon_press_cb(GtkWidget *widget, GdkEvent *event,
-				   SearchResult *r);
+static void     google_entry_activate_cb (GtkEntry *widget, Google *google);
+static gboolean google_icon_press_cb     (GtkWidget *widget, GdkEvent *event,
+					  SearchResult *r);
 
 /* create new google search item */
 GtkWidget *
@@ -69,6 +69,7 @@ google_new (KzWindow *kz)
 	
 	/* create google icon button */
 	button = gtk_button_new();
+	google->Icon = button;
 	gtk_widget_set_size_request(button, 20, 16);
 	gtk_button_set_relief((GtkButton *) button, GTK_RELIEF_NONE);
 	gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d(GTK_WIDGET(kz)->window,
@@ -185,7 +186,7 @@ google_parse_result(SearchResult *result)
 }
 #endif
 
-static void
+static gboolean
 google_icon_press_cb(GtkWidget *widget, GdkEvent *event,
 		     SearchResult *r)
 {
@@ -198,9 +199,12 @@ google_icon_press_cb(GtkWidget *widget, GdkEvent *event,
 				       menu_position_under_widget,
 				       widget,
 				       event_button->button, event_button->time);
+			return TRUE;
 		}
 	}
 #endif
+
+	return FALSE;
 }
 
 static void
