@@ -94,12 +94,12 @@ kz_gesture_dispose (GObject *object)
 
 	gesture = KZ_GESTURE(object);
 
-	kz_gesture_items_unref(gesture->items);
+	if (gesture->items)
+		kz_gesture_items_unref(gesture->items);
 	gesture->items = NULL;
 
-	if (G_OBJECT_CLASS (parent_class)->dispose) {
+	if (G_OBJECT_CLASS (parent_class)->dispose)
 		G_OBJECT_CLASS (parent_class)->dispose(object);
-	}
 }
 
 KzGesture *
