@@ -30,6 +30,7 @@
 static void kz_moz_embed_class_init    (KzMozEmbedClass *class);
 static void kz_moz_embed_init          (KzMozEmbed *kzembed);
 static void kz_moz_embed_destroy       (GtkObject *object);
+static void kz_moz_embed_finalize      (GObject    *object);
 static void kz_moz_embed_title         (GtkMozEmbed *embed);
 static void kz_moz_embed_location      (GtkMozEmbed *embed);
 static void kz_moz_embed_net_start     (GtkMozEmbed *embed);
@@ -43,7 +44,9 @@ static gint kz_moz_embed_dom_key_down  (GtkMozEmbed *embed, gpointer event);
 static gint kz_moz_embed_dom_key_up    (GtkMozEmbed *embed, gpointer event);
 static gint kz_moz_embed_dom_key_press (GtkMozEmbed *embed, gpointer event);
 
-void mozilla_setup();
+static gboolean mozilla_prefs_init (void);
+static void     mozilla_prefs_set  (void);
+static void     mozilla_setup      (void);
 
 static GtkMozEmbedClass *parent_class = NULL;
 
@@ -54,6 +57,7 @@ KZ_OBJECT_GET_TYPE(kz_moz_embed, "KzMozEmbed", KzMozEmbed,
 static void
 kz_moz_embed_class_init (KzMozEmbedClass *class)
 {
+	GObjectClass *gobject_class;
 	GtkObjectClass *object_class;
 	GtkMozEmbedClass *moz_embed_class;
 
@@ -61,11 +65,15 @@ kz_moz_embed_class_init (KzMozEmbedClass *class)
 
 	parent_class = g_type_class_peek_parent (class);
 
+	gobject_class = (GObjectClass *) class;
 	object_class = (GtkObjectClass *) class;
 	moz_embed_class = (GtkMozEmbedClass *) class;
 
+	/* GObject */
+	gobject_class->finalize = kz_moz_embed_finalize;
+
 	/* GtkObject signals */
-	object_class->destroy = kz_moz_embed_destroy;
+	object_class->destroy  = kz_moz_embed_destroy;
 
 	/* GtkMozEmbedSignals */
 #if 0
@@ -77,7 +85,6 @@ kz_moz_embed_class_init (KzMozEmbedClass *class)
 	moz_embed_class->progress_all    = kz_moz_embed_progress_all;
 	moz_embed_class->js_status       = kz_moz_embed_js_status;
 	moz_embed_class->destroy_browser = kz_moz_embed_destroy_browser;
-	moz_embed_class->size_to         = kz_moz_embed_size_to;
 #endif
 
 	moz_embed_class->title           = kz_moz_embed_title;
@@ -85,6 +92,7 @@ kz_moz_embed_class_init (KzMozEmbedClass *class)
 	moz_embed_class->net_start       = kz_moz_embed_net_start;
 	moz_embed_class->net_stop        = kz_moz_embed_net_stop;
 	moz_embed_class->open_uri        = kz_moz_embed_open_uri;
+	moz_embed_class->size_to         = kz_moz_embed_size_to;
 	moz_embed_class->visibility      = kz_moz_embed_visibility;
 	moz_embed_class->dom_key_press   = kz_moz_embed_dom_key_press;
 	moz_embed_class->dom_key_down    = kz_moz_embed_dom_key_down;
@@ -95,7 +103,6 @@ static void
 kz_moz_embed_init (KzMozEmbed *kzembed)
 {
 	/* widgets */
-	kzembed->parent_kazehakase     = NULL;
 	kzembed->popup_window          = NULL;
 
 	/* status */
@@ -114,11 +121,10 @@ kz_moz_embed_init (KzMozEmbed *kzembed)
 }
 
 GtkWidget *
-kz_moz_embed_new (KzWindow *kz, const gchar *url)
+kz_moz_embed_new (const gchar *url)
 {
 	KzMozEmbed *kzembed = KZ_MOZ_EMBED(g_object_new (KZ_TYPE_MOZ_EMBED, NULL));
 
-	kzembed->parent_kazehakase = kz;
 #if 0
 	/*
 	 * set the chrome type so it's stored in the object
@@ -126,7 +132,8 @@ kz_moz_embed_new (KzWindow *kz, const gchar *url)
 	g_moz_embed_set_chrome_mask(GTK_MOZ_EMBED(kzembed->mozembed),
 				    actualChromeMask);
 #endif
-	gtk_moz_embed_load_url(GTK_MOZ_EMBED(kzembed), url);
+	if (url)
+		gtk_moz_embed_load_url(GTK_MOZ_EMBED(kzembed), url);
 
 	return GTK_WIDGET (kzembed);
 }
@@ -152,6 +159,13 @@ kz_moz_embed_destroy (GtkObject *object)
 		GTK_OBJECT_CLASS (parent_class)->destroy(object);
 }
 
+static void
+kz_moz_embed_finalize (GObject *object)
+{
+	if (G_OBJECT_CLASS (parent_class)->finalize)
+		G_OBJECT_CLASS (parent_class)->finalize(object);
+}
+
 const gchar *
 kz_moz_embed_get_title (KzMozEmbed *kzembed)
 {
@@ -228,13 +242,12 @@ kz_moz_embed_open_uri (GtkMozEmbed *embed, const char *uri)
 	return FALSE;
 }
 
-#if 0
 static void
 kz_moz_embed_size_to (GtkMozEmbed *embed, gint width, gint height)
 {
 	gtk_widget_set_size_request(GTK_WIDGET(embed), width, height);
+	gtk_widget_queue_resize(GTK_WIDGET(embed));
 }
-#endif
 
 static gint
 kz_moz_embed_dom_key_down (GtkMozEmbed *embed, gpointer event)
@@ -245,26 +258,6 @@ kz_moz_embed_dom_key_down (GtkMozEmbed *embed, gpointer event)
 static gint
 kz_moz_embed_dom_key_up (GtkMozEmbed *embed, gpointer event)
 {
-	if (!prefs.typeahead)
-	{
-		KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);
-		g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
-
-		KzWindow *kz = KZ_MOZ_EMBED(kzembed)->parent_kazehakase;
-		gulong key_code;
-		glong type;
-		KeyEventInfo info;
-		
-		mozilla_get_key_code(event, &key_code);
-		
-		type = mozilla_get_key_event_info(embed, event,&info);
-
-		if ((key_code == 191) && !(type & CONTEXT_INPUT)) /* '/' */
-		{
-			gtk_widget_grab_focus(kz->findArea);
-		}
-	}
-
 	return FALSE;
 }
 
@@ -278,6 +271,7 @@ static void
 kz_moz_embed_visibility (GtkMozEmbed *embed, gboolean visibility)
 {
 	GtkWidget *parent = NULL;
+
 	parent = gtk_widget_get_parent(GTK_WIDGET(embed));
 	g_return_if_fail(parent != NULL);
 	
@@ -305,7 +299,8 @@ new_window_orphan_cb (GtkMozEmbedSingle *embed, GtkMozEmbed **retval,
 {
 	/* FIXME! but I don't have any nice idea... */
 	GtkWidget *newWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	GtkMozEmbed *newBrowser = GTK_MOZ_EMBED(kz_moz_embed_new(NULL, NULL));
+	GtkMozEmbed *newBrowser = GTK_MOZ_EMBED(kz_moz_embed_new(NULL));
+
 	gtk_container_add(GTK_CONTAINER(newWindow), GTK_WIDGET(newBrowser));
 	g_signal_connect(G_OBJECT(newBrowser),"destroy_browser",
 			 G_CALLBACK(destroy_browser_cb),newWindow);
@@ -313,11 +308,11 @@ new_window_orphan_cb (GtkMozEmbedSingle *embed, GtkMozEmbed **retval,
 }
 
 static gboolean
-mozilla_prefs_init()
+mozilla_prefs_init (void)
 {
 	const char *home_path;
 	char full_path[256];
-	
+
      	home_path = g_get_home_dir();
 	if (!home_path) {
 		g_error("Failed to get HOME");
@@ -332,27 +327,27 @@ mozilla_prefs_init()
 }
 
 static void 
-mozilla_prefs_set()
+mozilla_prefs_set (void)
 {
 	/* set user agent name */
 	mozilla_preference_set ("general.useragent.override",
 				prefs.user_agent);
 	/* set typeahead or not  */
 	mozilla_preference_set_boolean ("accessibility.typeaheadfind",
-				prefs.typeahead);	
+					prefs.typeahead);	
 }
 
-void
-mozilla_setup()
+static void
+mozilla_setup (void)
 {
 	GtkMozEmbedSingle *single = NULL;
-	
+
 	mozilla_prefs_init();
-	
+
 	gtk_moz_embed_push_startup();
 
 	mozilla_prefs_set();
-	
+
 	/*
 	 * get the singleton object and hook up to its new window callback
 	 * so we can create orphaned windows.
