@@ -22,10 +22,10 @@
 #define __KZ_WINDOW_H__
 
 #include <gtk/gtk.h>
-#include <gtkmozembed.h>
 
 #include "egg-action-group.h"
 #include "egg-menu-merge.h"
+#include "kz-mozembed.h"
 #include "kz-gesture.h"
 
 G_BEGIN_DECLS
@@ -42,53 +42,52 @@ G_BEGIN_DECLS
 #define KZ_WINDOW_CURRENT_PAGE(kz) \
 	(KZ_IS_WINDOW(kz) ? KZ_WINDOW_NTH_PAGE(kz, gtk_notebook_get_current_page(GTK_NOTEBOOK(kz->notebook))) : NULL)
 
-#define KZ_WINDOW_SET_SENSITIVE(kz, action_name, sensitive) \
-{ \
-	if (KZ_IS_WINDOW(kz) && EGG_IS_ACTION_GROUP(kz->action_group)) { \
-		EggAction *action; \
-		action = egg_action_group_get_action(kz->action_group, \
-						     action_name); \
-		if (action) egg_action_set_sensitive(action, sensitive); \
-	} \
+#define KZ_WINDOW_SET_SENSITIVE(kz, action_name, sensitive)			\
+{										\
+	if (KZ_IS_WINDOW(kz) && EGG_IS_ACTION_GROUP(kz->action_group)) { 	\
+		EggAction *action;						\
+		action = egg_action_group_get_action(kz->action_group,		\
+						     action_name);		\
+		if (action) egg_action_set_sensitive(action, sensitive);	\
+	}									\
 }
 
-#define KZ_WINDOW_SET_VISIBLE(kz, action_name, visible) \
-{ \
-	if (KZ_IS_WINDOW(kz) && EGG_IS_ACTION_GROUP(kz->action_group)) { \
-		EggAction *action; \
-		action = egg_action_group_get_action(kz->action_group, \
-						     action_name); \
-		if (action) egg_action_set_visible(action, visible); \
-	} \
+#define KZ_WINDOW_SET_VISIBLE(kz, action_name, visible)				\
+{										\
+	if (KZ_IS_WINDOW(kz) && EGG_IS_ACTION_GROUP(kz->action_group)) {	\
+		EggAction *action;						\
+		action = egg_action_group_get_action(kz->action_group,		\
+						     action_name);		\
+		if (action) egg_action_set_visible(action, visible);		\
+	}									\
 }
 
-typedef struct _KzWindow KzWindow;
-typedef struct _KzWindowClass KzWindowClass;
+typedef struct _KzWindow	KzWindow;
+typedef struct _KzWindowClass	KzWindowClass;
+typedef struct _KzWindowPriv	KzWindowPriv;
 
 struct _KzWindow
 {
 	GtkWindow   parent;
 
-	GtkWidget  *topLevelVBox;
+	GtkWidget  *top_vbox;
 
-	GtkWidget  *urlEntry;
-	GtkWidget  *progressAreaHBox;
-	GtkWidget  *progressBar;
-	GtkWidget  *statusAlign;
-	GtkWidget  *statusBar;
+	GtkWidget  *url_entry;
+	GtkWidget  *progress_area_hbox;
+	GtkWidget  *progressbar;
+	GtkWidget  *status_align;
+	GtkWidget  *statusbar;
 	GtkWidget  *notebook;
-	GtkWidget  *findAreaHBox;
-	GtkWidget  *findArea;
-	GtkWidget  *kztoolBar; /* Kazehakase tool bar */
+	GtkWidget  *find_area_hbox;
+	GtkWidget  *find_area;
+	GtkWidget  *kztoolbar; /* Kazehakase tool bar */
 
 	GtkAccelGroup  *accel_group;
 	EggActionGroup *action_group;
 	EggMenuMerge   *menu_merge;
 	KzGestureItems *gesture_items;
-	
-	const char *statusMessage;
-	char       *tempMessage;
-	gboolean   didFind; /* for find keyword */
+
+	KzWindowPriv *priv;
 };
 
 struct _KzWindowClass
@@ -96,15 +95,18 @@ struct _KzWindowClass
 	GtkWindowClass parent_class;
 };
 
-GtkType           kz_window_get_type           (void);
+GtkType	          kz_window_get_type           (void);
 GList            *kz_window_get_window_list    (void);
 GtkWidget        *kz_window_new                (const gchar *url);
+GtkWidget        *kz_window_get_from_tab       (GtkWidget *tab_widget);
 GtkWidget        *kz_window_open_new_tab       (KzWindow *kz,
 						const gchar *url);
 void              kz_window_close_tab          (KzWindow *kz,
 						GtkWidget *widget);
 void              kz_window_close_tab_idle     (KzWindow *kz,
 						GtkWidget *widget);
+void              kz_window_move_tab           (KzWindow *kz,
+						GtkWidget *widget);
 void              kz_window_load_url           (KzWindow *kz,
 						const gchar *url);
 void              kz_window_set_visibility     (KzWindow *kz,
